This patch allows you to override the primary VID/DID, subsystem VID/DID
PCI device class, and combined PCI revision and prog-if.

Using this is generally a really bad idea unless you know exactly what you're
doing; an example of a device that this is usable with is the HighPoint 1144CM
which is actually 4 ASmedia 1042 XHCI controllers behind a PCI-e switch,
reporting a "fake" PCI type as a RAID controller. Also might be useful for
passing through NVidia cards without hard modifying them.

diff -Nur qemu-2.5.0.orig/hw/vfio/pci.c qemu-2.5.0/hw/vfio/pci.c
--- qemu-2.5.0.orig/hw/vfio/pci.c	2015-12-16 14:04:49.000000000 -0800
+++ qemu-2.5.0/hw/vfio/pci.c	2016-03-09 15:40:08.224044362 -0800
@@ -2498,6 +2498,32 @@
     memset(&vdev->pdev.config[PCI_BASE_ADDRESS_0], 0, 24);
     memset(&vdev->pdev.config[PCI_ROM_ADDRESS], 0, 4);
 
+    if (vdev->virtual_vendor_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_VENDOR_ID,
+                               cpu_to_le16(vdev->virtual_vendor_id), 0xFFFF);
+    }
+    if (vdev->virtual_device_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_DEVICE_ID,
+                               cpu_to_le16(vdev->virtual_device_id), 0xFFFF);
+    }
+    if (vdev->virtual_ss_vendor_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_SUBSYSTEM_VENDOR_ID,
+                               cpu_to_le16(vdev->virtual_ss_vendor_id), 0xFFFF);
+    }
+    if (vdev->virtual_ss_device_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_SUBSYSTEM_ID,
+                               cpu_to_le16(vdev->virtual_ss_device_id), 0xFFFF);
+    }
+    if (vdev->virtual_class != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_CLASS_DEVICE,
+                               cpu_to_le16(vdev->virtual_class), 0xFFFF);
+    }
+    if (vdev->virtual_rev_and_prog_if != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_CLASS_REVISION,
+                               cpu_to_le16(vdev->virtual_rev_and_prog_if), 0xFFFF);
+    }
+
+
     vfio_pci_size_rom(vdev);
 
     ret = vfio_msix_early_setup(vdev);
@@ -2626,6 +2652,12 @@
                        intx.mmap_timeout, 1100),
     DEFINE_PROP_BIT("x-vga", VFIOPCIDevice, features,
                     VFIO_FEATURE_ENABLE_VGA_BIT, false),
+    DEFINE_PROP_UINT16("x-vid", VFIOPCIDevice, virtual_vendor_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-did", VFIOPCIDevice, virtual_device_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-ss-vid", VFIOPCIDevice, virtual_ss_vendor_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-ss-did", VFIOPCIDevice, virtual_ss_device_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-class", VFIOPCIDevice, virtual_class, 0xFFFF),
+    DEFINE_PROP_UINT16("x-rev-and-progif", VFIOPCIDevice, virtual_rev_and_prog_if, 0xFFFF),
     DEFINE_PROP_BIT("x-req", VFIOPCIDevice, features,
                     VFIO_FEATURE_ENABLE_REQ_BIT, true),
     DEFINE_PROP_BOOL("x-no-mmap", VFIOPCIDevice, vbasedev.no_mmap, false),
diff -Nur qemu-2.5.0.orig/hw/vfio/pci.h qemu-2.5.0/hw/vfio/pci.h
--- qemu-2.5.0.orig/hw/vfio/pci.h	2015-12-16 14:04:49.000000000 -0800
+++ qemu-2.5.0/hw/vfio/pci.h	2016-03-09 15:41:16.028506221 -0800
@@ -128,6 +128,12 @@
 #define VFIO_FEATURE_ENABLE_REQ_BIT 1
 #define VFIO_FEATURE_ENABLE_REQ (1 << VFIO_FEATURE_ENABLE_REQ_BIT)
     int32_t bootindex;
+    uint16_t virtual_vendor_id;
+    uint16_t virtual_device_id;
+    uint16_t virtual_ss_vendor_id;
+    uint16_t virtual_ss_device_id;
+    uint16_t virtual_class;
+    uint16_t virtual_rev_and_prog_if;
     uint8_t pm_cap;
     bool has_vga;
     bool pci_aer;
